/*

This code was written by Kevin King for Plum Geek LLC.
Portions from other open source projects where noted.
This code is licensed under:
Creative Commons Attribution-ShareAlike 2.0 Generic (CC BY-SA 2.0)
https://creativecommons.org/licenses/by-sa/2.0/
Visit http://www.plumgeek.com for Plumduino information.
Visit http://www.arduino.cc to learn about the Arduino.

*/

#ifndef PLUMDUINO_HARDWARE_H
#define PLUMDUINO_HARDWARE_H

#include "Arduino.h"
#include <FastLED.h>
#include <Adafruit_NeoPixel.h>
#include <EEPROM.h>


// ***************************************************
// Pin defines
// ***************************************************

#define SenseAmbient A2
#define SenseLeft A0
#define SenseRight A1
#define IRSource 5
#define Button 2
#define Light_Bus 4
#define LED 13        //orange general purpose LED on right side of board
#define PIN 4         //pixel pin, used for AdaFruit NeoPixel Libraray
#define DATA_PIN 4    //pixel pin, used for FastLED Library

//Pins for Plumduino Expansion Board
#define Slide1 A4
#define Slide2 A5
#define Chirp 9
#define Beeper 9      //Wink sketches define this pin as "Beeper"
#define Mic A3
#define Motion 8
#define IR_Send 10
#define _38kHz_Rx 3

// ***************************************************
// end Pin defines
// ***************************************************



// ***************************************************
// General hardware
// ***************************************************
#define SERIAL_SPEED 57600    //serial speed used for Serial Monitor window
#define NUM_PIXELS  8         //used for Adafruit_NeoPixel library
#define NUM_LEDS    8         //used for the FastLED library 
#define PROXFEEDBACK 35       //the amount of feedback to the proximity sensors. 35 is about normal.
#define MIC_CENTER 350        //center "quiet" reading of microphone

Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUM_PIXELS, Light_Bus, NEO_GRB + NEO_KHZ800);  //start traditional NeoPixel

extern void hardwareBegin(void);

// ***************************************************
// end General hardware
// ***************************************************



// ***************************************************
// Plumduino Global Variables Initialization
// ***************************************************

int proxLeft, proxRight;    //proximity sensor readings
int red,green,blue;         //for results of getColorWheel() function
CRGB leds[NUM_LEDS];        //array to hold FastLED pixel data


// ***************************************************
// End of Plumduino Global Variables Initialization
// ***************************************************


// ***************************************************
// IR
// ***************************************************

//The following was used as reference code in the development of the IR receive functions:
//http://playground.arduino.cc/Code/InfraredReceivers by Paul Malmsten 
//https://github.com/z3t0/Arduino-IRremote by Ken Shirriff
extern void TxIR(unsigned char *Data, int Length);
extern void TxIRKey(byte key);
//
extern void RxIRStop(void);
extern void RxIRRestart(char BytesToLookFor);
extern char IsIRDone(void);
extern byte GetIRButton(void);
extern char IRNumOfBytes;
extern char CheckMenuButton(void);
//
extern byte GetIRButton(void);
extern const uint8_t IRRemoteButtons[][2];
#define IR_1 1
#define IR_2 2
#define IR_3 3
#define IR_4 4
#define IR_5 5
#define IR_6 6
#define IR_7 7
#define IR_8 8
#define IR_9 9
#define IR_0 10
#define IR_Forward 11
#define IR_Left 12
#define IR_Right 13
#define IR_Backward 14
#define IR_Power 15
#define IR_PlumLogo 16
#define IR_Menu 17
#define IR_A 18
#define IR_B 19
#define IR_Play 20
#define IR_X 21
//


//Low level functions:
extern int IRTransitionCount;
extern unsigned char IRBytes[20];
extern byte irData[]={0x00,0xFF,0x00,0x00};
extern char IRActive;
extern volatile char IRReceiving;//note: IRReceiving turned off if IsIRDone() in regular or auto NavigationHandler() and causes ReadSideSensors() to repeat
//
//extern void EnableIROutputs(char Level);  //enables or disables the 3 IR outputs. Pass 0 to disable, 1 to enable all 3 IR outputs.
extern void ModulateIR(unsigned int Frequency, unsigned int OnTime); //ModulateIR(38000,5) seems to produce best square wave for 38kHz.
//extern void PlayChirpIR(unsigned int Frequency, unsigned int OnTime); //wrapper for IRCarrierWave, bkwd compatibility. use ModulateIR() instead of PlayChirpIR going forward
//
extern void IRHandler(void);
//
// ***************************************************
// end IR
// ***************************************************


// ***************************************************
// EEPROM Functions
// ***************************************************

//Copied from http://playground.arduino.cc/Code/EEPROMWriteAnything  Visit page for use instructions.
#include <EEPROM.h>
#include <Arduino.h>  // for type definitions

template <class T> int EEPROM_writeAnything(int ee, const T& value)
{
    const byte* p = (const byte*)(const void*)&value;
    unsigned int i;
    for (i = 0; i < sizeof(value); i++)
          EEPROM.write(ee++, *p++);
    return i;
}

template <class T> int EEPROM_readAnything(int ee, T& value)
{
    byte* p = (byte*)(void*)&value;
    unsigned int i;
    for (i = 0; i < sizeof(value); i++)
          *p++ = EEPROM.read(ee++);
    return i;
}

// ***************************************************
// end EEPROM Functions
// ***************************************************

#endif
