/*

This code was written by Kevin King for Plum Geek LLC.
Portions from other open source projects where noted.
This code is licensed under:
Creative Commons Attribution-ShareAlike 2.0 Generic (CC BY-SA 2.0)
https://creativecommons.org/licenses/by-sa/2.0/
Visit http://www.plumgeek.com for Plumduino information.
Visit http://www.arduino.cc to learn about the Arduino.

*/

#include "PlumduinoHardware.h"
#include <FastLED.h>
#include <Adafruit_NeoPixel.h>
#include <EEPROM.h>

// ***************************************************
// Plumduino Hardware Initialization
// ***************************************************

void hardwareBegin(void){//Ver. 1.0, Kevin King

  pinMode(IRSource, OUTPUT);
  digitalWrite(IRSource, LOW);
  pinMode(IR_Send, OUTPUT);
  digitalWrite(IR_Send, LOW);
  pinMode(SenseLeft, INPUT);
  pinMode(SenseRight, INPUT);
  pinMode(SenseAmbient, INPUT);
  pinMode(LED, OUTPUT);
  digitalWrite(LED, LOW);
  Serial.begin(SERIAL_SPEED);
   
  FastLED.addLeds<NEOPIXEL, DATA_PIN>(leds, NUM_LEDS);  //start FastLED
                                                        //FastLED Tutorial: https://github.com/FastLED/FastLED/wiki/Basic-usage
                                                        //FastLED Docs: http://fastled.io/docs/3.1/class_c_fast_l_e_d.html
  FastLED.clear();
  FastLED.show();
}

// ***************************************************
// End of Plumduino Hardware Initialization
// ***************************************************



char buttonPressed(void){//Ver. 1.0, Dustin Soodak
  return (digitalRead(Button)==1);  // returns 1 (or "true") if button is being pressed. 
                                    // returns 0 (or "false") if not pressed.
}

void waitForButton(void){  // Ver 1.0, Kevin King
  while(!buttonPressed()){          // waits for button to be pressed
    //do nothing (while waiting for button press)
  }
}

void playChirp(unsigned int Frequency){//Ver. 1.0, Dustin Soodak
  //Frequency is in Hz
  //Amplitude is in units of UsOn
  int Amplitude = 50;
  uint16_t temp;
  uint16_t period,dutycycle;
  uint8_t prescalerbits;
  if(Amplitude>100)
    Amplitude=100;
  if (F_CPU/Frequency/2>=0x10000){
    if(F_CPU/Frequency/2>=0x10000*8){
        prescalerbits=0b011;//prescaler 64
        period=F_CPU/Frequency/(2*16);
        dutycycle=F_CPU/1000000*Amplitude/2/64;               
    }
    else{
      prescalerbits = 0b010;// prescaler 8
      period=F_CPU/Frequency/(2*8);
      dutycycle=F_CPU/1000000*Amplitude/2/8;
    }
  }
  else{
    prescalerbits = 0b001;  //on but no prescaling
    period=F_CPU/Frequency/(2*1);
    dutycycle=F_CPU/1000000*Amplitude/2/1;
  }
  if(Amplitude==0){
    TCCR1A=0;
    TCCR1B=0;
    pinMode(Chirp,INPUT);
    
    //Serial.println("off");    // debugging code
  }
  else{
    TCCR1B&=~0b00000111;//turn off timer
    ICR1=period;
    OCR1A=dutycycle;
    TCCR1A = (0b10<<6) | 0b10;//COM1A1 COM1A0, and WGM11 WGM10
    TCCR1B = (0b10<<3) | prescalerbits;//WGM13 WGM12, and off/(on with prescaler)
    pinMode(Chirp,OUTPUT);//DDRB|=1;//make PortB pin1 an output 
  }
}

void offChirp(void){//Ver. 1.0, Kevin King              //stops chirp tone
  noTone(Chirp);
  //PlayChirp(0);       //turn off chirp
}

void setPixelRGB(int Pixel, int Red, int Green, int Blue){//Ver. 1.0, Dustin Soodak
  //if(Pixel>7)
  //  Pixel=1;
  //if(Pixel<0)
  //  Pixel=0;
  pixels.setPixelColor(Pixel, pixels.Color(Red,Green,Blue));
  pixels.show();
}

void setAllPixelsRGB(int Red, int Green, int Blue){//Ver. 1.1, Dustin Soodak
  char i;
  for(i=0;i<8;i++){
    pixels.setPixelColor(i, pixels.Color(Red,Green,Blue)); 
  }
  pixels.show();//added Ver.1.1
}

void pixelRed(int pixel, byte brightness){
  setPixelRGB(pixel,brightness,0,0);
}

void pixelGreen(int pixel, byte brightness){
  setPixelRGB(pixel,0,brightness,0);
}

void pixelBlue(int pixel, byte brightness){
  setPixelRGB(pixel,0,0,brightness);
}

void pixelPurple(int pixel, byte brightness){
  byte n;
  n=brightness/2;
  setPixelRGB(pixel,n,0,brightness);
}

void pixelPink(int pixel, byte brightness){
  byte n;
  n=brightness/2;
  setPixelRGB(pixel,brightness,0,n);
}

void pixelYellow(int pixel, byte brightness){
  setPixelRGB(pixel,brightness,brightness,0);
}

void pixelOrange(int pixel, byte brightness){
  byte n;
  n=brightness/4;
  setPixelRGB(pixel,brightness,n,0);
}

void pixelCyan(int pixel, byte brightness){
  setPixelRGB(pixel,0,brightness,brightness);
}

void pixelWhite(int pixel, byte brightness){
  setPixelRGB(pixel,brightness,brightness,brightness);
}


void offPixels(void){
  clearStrip();
}

void clearStrip() {
  for( int i = 0; i<NUM_PIXELS; i++){
    setPixelRGB(i,0,0,0);
    //pixels.setPixelColor(i, 0x000000); pixels.show();
  }
  //pixels.show();
}

int micAmplitude(void){   //returns amplitude of Mic signal
  int amp;
  amp = analogRead(Mic);
  amp = amp - MIC_CENTER;
  amp = abs(amp);
  return amp;
}

void readProximity(void){
  digitalWrite(IRSource, HIGH);
  delay(2);
  proxLeft = analogRead(SenseLeft);
  proxRight = analogRead(SenseRight);
  digitalWrite(IRSource, LOW);
  delay(2);
  proxLeft = proxLeft - analogRead(SenseLeft) - PROXFEEDBACKLEFT;
  proxRight = proxRight - analogRead(SenseRight) - PROXFEEDBACKRIGHT;
  proxLeft = constrain(proxLeft, 0, 1024);
  proxRight = constrain(proxRight, 0, 1024);
}

void tab(void){
  Serial.print("\t");
}

void nl(void){
  Serial.println("");
}


//Convert a given hue and brightness to RGB(Red Green Blue) and populate the
//global variables red, green, and blue with the result which can be directly
//sent to the pixels via setPixelRGB() and the like.
//This function will automatically "unwrap" full rotations.
//  h is hue value, integer between 0 and 360
//  bright is the brightness value between 0 and 255
//  Based on code by Chris Hulbert, found at: http://splinter.com.au/blog/?p=29
void getColorWheel(int h, double bright) {

  //make sure value passed falls within 0 and 360
  if (h > 359){
    while (h > 359){
      h = h - 359;  // subtract full rotations until within range
    }
  }

  if (h < 0){
    while (h < 0){
      h = h + 359;  // add full rotations until within range
    }
    h = 359 - h;
  }

  //this is the algorithm to convert from RGB to HSV
  double r=0; 
  double g=0; 
  double b=0;
  double s=1;
  double v = bright / 255;

  double hf=h/60.0;

  int i=(int)floor(h/60.0);
  double f = h/60.0 - i;
  double pv = v * (1 - s);
  double qv = v * (1 - s*f);
  double tv = v * (1 - s * (1 - f));

  switch (i)
  {
  case 0: //rojo dominante
    r = v;
    g = tv;
    b = pv;
    break;
  case 1: //verde
    r = qv;
    g = v;
    b = pv;
    break;
  case 2: 
    r = pv;
    g = v;
    b = tv;
    break;
  case 3: //azul
    r = pv;
    g = qv;
    b = v;
    break;
  case 4:
    r = tv;
    g = pv;
    b = v;
    break;
  case 5: //rojo
    r = v;
    g = pv;
    b = qv;
    break;
  }

  //set each component to a integer value between 0 and 255
  red=constrain((int)255*r,0,255);
  green=constrain((int)255*g,0,255);
  blue=constrain((int)255*b,0,255);

}


// ***************************************************
// IR
// ***************************************************

void TxIR(unsigned char *Data, int Length){//Ver. 1.2, Dustin Soodak
    int i;
    char j;
    const uint16_t Freq=38000,UsOn=10; //For R2=2k pull up, 8 us delay before pin falls. Inputs (28000,5) give a decent square wave in this case. 
    RxIRStop();
    //EnableIROutputs(1);
    ModulateIR(Freq,UsOn);    //start sending 38 kHz carrier
    delayMicroseconds(9000);
    //EnableIROutputs(0);               //EnableIROutputs(0) turns off IR 38khz out but this makes receiver voltage go high.
    ModulateIR(Freq, 0);      //stop sending 38 kHz carrier
    delayMicroseconds(4500);
    //EnableIROutputs(1);
    ModulateIR(Freq,UsOn);    //start sending 38 kHz carrier
    delayMicroseconds(455); //520
    for(i=0;i<Length;i++){      
      for(j=0;j<8;j++){
        //EnableIROutputs(0);
        ModulateIR(Freq, 0);      //stop sending 38 kHz carrier
        if((Data[i])&(1<<j))
          delayMicroseconds(1620);  //1610
        else
          delayMicroseconds(510);   //580
        //EnableIROutputs(1);
        ModulateIR(Freq,UsOn);    //start sending 38 kHz carrier
        delayMicroseconds(455);  //520
      }      
    }//end for(i=0;i<Length;i++)    
    //ModulateIR(38000, 0);
    ModulateIR(Freq, 0);      //stop sending 38 kHz carrier
    //EnableIROutputs(1); 
}//end TxIR()

void ModulateIR(unsigned int Frequency, unsigned int OnTime){//Ver. 1.0, Dustin Soodak 
  //ModulateIR(38000,6) seems to produce best square wave for 38kHz.
  //Frequency is in Hz
  //OnTime is in units of UsOn
  uint16_t temp;
  uint16_t period,dutycycle;
  uint8_t prescalerbits;
  if(OnTime>100)
    OnTime=100;
  if (F_CPU/Frequency/2>=0x10000){
    if(F_CPU/Frequency/2>=0x10000*8){
        prescalerbits=0b011;//prescaler 64
        period=F_CPU/Frequency/(2*16);
        dutycycle=F_CPU/1000000*OnTime/2/64;               
    }
    else{
      prescalerbits = 0b010;// prescaler 8
      period=F_CPU/Frequency/(2*8);
      dutycycle=F_CPU/1000000*OnTime/2/8;
    }
  }
  else{
    prescalerbits = 0b001;  //on but no prescaling
    period=F_CPU/Frequency/(2*1);
    dutycycle=F_CPU/1000000*OnTime/2/1;
  }
  if(OnTime==0){
    TCCR1A=0;
    TCCR1B=0;
    pinMode(Beeper,INPUT);
    
    //Serial.println("off");
  }
  else{
    TCCR1B&=~0b00000111;//turn off timer    
    ICR1=period;
    OCR1B=dutycycle;
    TCCR1A = (0b10<<4) | 0b10;//COM1B1 COM1B0, and WGM11 WGM10
    TCCR1B = (0b10<<3) | prescalerbits;//WGM13 WGM12, and off/(on with prescaler)    
  }
}

void TxIRKey(byte key){//Ver. 1.0, Kevin King
  if(key<1){
    // do nothing
  }
  else if(key>21){
    // do nothing
  }
  else{     // actually send IR key if it was within correct range
  key-=1;   //subract 1 from key number so it matches array IRRemoteButtons
  irData[0]=0x00;irData[1]=0xFF;     // all remote keys begin with 0x00, 0xFF
  irData[2]=IRRemoteButtons[key][0]; // add 3rd value
  irData[3]=IRRemoteButtons[key][1]; // add 4th value
  TxIR(irData,4);                    // actually transmit via any enabled IR sources
  }
}//end TxIRKey()

int IRTransitionCount=0;
char IRBitNum=0,IRByte=0,IRNumOfBytes=0;
unsigned char IRBytes[20];
uint16_t IRPrevTime=0,IRTime;
uint32_t MsAtLastIR=0;//used for end of communication timeout
volatile char IRReceiving=0;
char IRActive=0;
char IRMaxByteCount=4;


void RxIRRestart(char BytesToLookFor){//Ver. 1.2, Dustin Soodak
  int i;
  detachInterrupt(1);//interrupt 1 is I/O 3 which is _38kHz_R
  TCCR1A = 0x00;          // COM1A1=0, COM1A0=0 => Disconnect Pin OC1 from Timer/Counter 1 -- PWM11=0,PWM10=0 => PWM Operation disabled
  // ICNC1=0 => Capture Noise Canceler disabled -- ICES1=0 => Input Capture Edge Select (not used) -- CTC1=0 => Clear Timer/Counter 1 on Compare/Match
  // CS12=0 CS11=1 CS10=1 => Set prescaler to clock/64
  TCCR1B = 0x03;          // 8MHz clock with prescaler 0x03 means TCNT1 increments every 8uS
  // ICIE1=0 => Timer/Counter 1, Input Capture Interrupt Enable -- OCIE1A=0 => Output Compare A Match Interrupt Enable -- OCIE1B=0 => Output Compare B Match Interrupt Enable
  // TOIE1=0 => Timer 1 Overflow Interrupt Enable
  TIMSK1 = 0x00;          
  pinMode(_38kHz_Rx, INPUT_PULLUP);  
  IRReceiving=0;  
  IRTransitionCount=0;
  IRPrevTime=0;
  MsAtLastIR=0;IRBitNum=0;IRByte=0;IRNumOfBytes=0;
  IRActive=1;
  IRMaxByteCount=BytesToLookFor;
  attachInterrupt(1, IRHandler, CHANGE);//interrupt 1 is I/O 3 which is _38kHz_Rx  
}
void RxIRStop(void){//Ver. 1.2, Dustin Soodak
  detachInterrupt(1);//interrupt 1 is I/O 3 which is _38kHz_Rx    
  TCCR1B=0;//turn off timer        
  pinMode(_38kHz_Rx, INPUT_PULLUP);  
  
  MsAtLastIR=0;//so IRHandler() recognizes it as first falling edge of next transition
  IRReceiving=0;
  IRActive=0;
  IRTransitionCount=0;//so IsIRDone() does not expect anything just because RxIRStop() called.
}

// Infrared transmit and recieve functions are based on:
//http://playground.arduino.cc/Code/InfraredReceivers by Paul Malmsten 
//https://github.com/z3t0/Arduino-IRremote by Ken Shirriff

void IRHandler(void){//Ver. 2.0, Dustin Soodak, Kevin King
  //using interrupt 1 is I/O 3 which is _38kHz_Rx  
  int16_t dTime;
  char Level;
  noInterrupts();
  IRTime=TCNT1;
   
  Level=digitalRead(_38kHz_Rx);  

  if(!Level){//note; 38khz IR signal makes level go low
    IRReceiving=1;
  }
  if((millis()-MsAtLastIR>15) && (IRNumOfBytes<IRMaxByteCount)) {//should never be more than 9 inside valid signal
    TCNT1=0;
    IRPrevTime=0;
    IRTransitionCount=0;
    IRBitNum=0;IRNumOfBytes=0;IRByte=0;  
    //IRReceiving=1;  
  }  
  else{  
    if(IRTime>IRPrevTime)
      dTime=IRTime-IRPrevTime;
    else
      dTime=0xFFFF-IRPrevTime+1+IRTime;   
    IRPrevTime=IRTime;    
    dTime=dTime<<3;
    if(IRTransitionCount>=3 && (IRTransitionCount&1)){//should be high
      if(dTime>1000){
        IRByte|=(1<<IRBitNum); 
      }
      if(dTime<300){//error
         IRNumOfBytes=0;
         IRReceiving=0;
      }
      IRBitNum++;
      if(IRBitNum>7){
        if(IRNumOfBytes<IRMaxByteCount){
          IRBytes[IRNumOfBytes]=IRByte;
          IRNumOfBytes++;
        }        
        else{
           IRReceiving=0;           
        }
        IRBitNum=0; 
        IRByte=0;               
      }      
    }
    IRTransitionCount++;
  }
  MsAtLastIR=millis();
  interrupts();
}

char IsIRDone(void){//Ver. 1.0, Dustin Soodak
  return ((millis()-MsAtLastIR>40 || IRNumOfBytes==IRMaxByteCount) && IRTransitionCount);  
}

//Note: first and second bytes are always 0x00 and 0xFF. IRRemoteButtons[] contains the third and fourth bytes.
const byte IRRemoteButtons[][2]={
  {0x0C,0xF3},//"1" key: 1
  {0x18,0xE7},//"2" key: 2
  {0x5E,0xA1},//"3" key: 3
  {0x08,0xF7},//"4" key: 4
  {0x1C,0xE3},//"5" key: 5
  {0x5A,0xA5},//"6" key: 6
  {0x42,0xBD},//"7" key: 7
  {0x52,0xAD},//"8" key: 8
  {0x4A,0xB5},//"9" key: 9
  {0x16,0xE9},//"0" key: 10
  {0x40,0xBF},//"FORWARD" key: 11
  {0x07,0xF8},//"LEFT" key: 12
  {0x09,0xF6},//"RIGHT" key: 13
  {0x19,0xE6},//"BACKWARD" key: 14
  {0x45,0xBA},//"POWER" key: 15
  {0x46,0xB9},//"PLUM LOGO" key: 16
  {0x47,0xB8},//"MENU" key: 17
  {0x44,0xBB},//"A" key: 18
  {0x43,0xBC},//"B" key: 19
  {0x15,0xEA},//"PLAY" key: 20
  {0x0D,0xF2}//"X" key: 21
};
byte GetIRButton(void){//Ver. 2.0, Dustin Soodak, Kevin King
  byte ButtonNumber=0,i;

    for(i=0;i<sizeof(IRRemoteButtons)/2;i++){
      if(IRBytes[2]==IRRemoteButtons[i][0] && IRBytes[3]==IRRemoteButtons[i][1]){
        ButtonNumber=i+1;
        break; 
      }
    } 

    return ButtonNumber;       //return the button number that was pressed
    
}// end GetIRButton()

// ***************************************************
// end IR
// ***************************************************


